ChargingStation = {}
local ChargingStation_mt = Class(ChargingStation, BuyingStationPlaceable)

InitObjectClass(ChargingStation, "ChargingStation")

function ChargingStation:new(isServer, isClient, customMt)
	local self = BuyingStationPlaceable:new(isServer, isClient, customMt or ChargingStation_mt)

	registerObjectClassName(self, "ChargingStation")

	return self
end

function ChargingStation:load(xmlFilename, x, y, z, rx, ry, rz, initRandom)
	if not ChargingStation:superClass().load(self, xmlFilename, x, y, z, rx, ry, rz, initRandom) then
		return false
	end

	local xmlFile = loadXMLFile("TempXML", xmlFilename)
	self.chargeIndicatorIntensity = getXMLFloat(xmlFile, "placeable.chargingStation.chargeIndicator#intensity") or 20
	self.chargeIndicatorBlinkSpeed = getXMLFloat(xmlFile, "placeable.chargingStation.chargeIndicator#blinkSpeed") or 5
	self.chargeIndicatorNode = I3DUtil.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.chargingStation.chargeIndicator#node"))

	if self.chargeIndicatorNode ~= nil then
		setShaderParameter(self.chargeIndicatorNode, "lightControl", self.chargeIndicatorIntensity, 0, 0, 0, false)
		setShaderParameter(self.chargeIndicatorNode, "emitColor", 1, 1, 0, 0, false)
	end

	self.chargeIndicatorLight = I3DUtil.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.chargingStation.chargeIndicator#light"))

	if self.chargeIndicatorLight then
		setLightColor(self.chargeIndicatorLight, 0, 0, 0)
	end

	self.chargeIndicatorColorFull = StringUtil.getVectorNFromString(getXMLString(xmlFile, "placeable.chargingStation.chargeIndicator#colorFull") or "0 1 0 1", 4)
	self.chargeIndicatorColorEmpty = StringUtil.getVectorNFromString(getXMLString(xmlFile, "placeable.chargingStation.chargeIndicator#colorEmpty") or "1 1 0 1", 4)
	self.chargeIndicatorLightColor = self.chargeIndicatorColorFull
	self.interactionRadius = getXMLFloat(xmlFile, "placeable.chargingStation#interactionRadius") or 5
	self.fillLitersPerMS = (getXMLFloat(xmlFile, "placeable.chargingStation#fillLitersPerSecond") or 1) / 1000
	self.loadTrigger = nil

	if self.buyingStation ~= nil then
		for j = 1, #self.buyingStation.loadTriggers do
			local loadTrigger = self.buyingStation.loadTriggers[j]
			loadTrigger.startFillText = g_i18n:getText("action_startCharging")
			loadTrigger.stopFillText = g_i18n:getText("action_stopCharging")
			loadTrigger.activateText = loadTrigger.startFillText
			loadTrigger.fillLitersPerMS = self.fillLitersPerMS
			self.loadTrigger = loadTrigger
			self.fillSample = g_soundManager:loadSampleFromXML(xmlFile, "placeable.chargingStation.sounds", "fill", self.baseDirectory, self.nodeId, 0, AudioGroup.ENVIRONMENT, nil, nil)

			if self.fillSample ~= nil and loadTrigger.samples.load == nil then
				loadTrigger.samples.load = self.fillSample
			end
		end

		local mapHotspot = self.buyingStation.mapHotspot

		mapHotspot:setBorderedImage(self.baseDirectory .. "hud_battery_icon.png", Overlay.DEFAULT_UVS)
	end

	delete(xmlFile)

	return true
end

function ChargingStation:getChargeState()
	if self.loadTrigger ~= nil then
		local index = next(self.loadTrigger.fillableObjects)

		if index ~= nil then
			local vehicle = self.loadTrigger.fillableObjects[index].object

			if vehicle.getConsumerFillUnitIndex ~= nil then
				local fillUnitIndex = vehicle:getConsumerFillUnitIndex(FillType.ELECTRICCHARGE)

				if fillUnitIndex ~= nil then
					local fillLevel = vehicle:getFillUnitFillLevel(fillUnitIndex)
					local capacity = vehicle:getFillUnitCapacity(fillUnitIndex)

					return fillLevel, capacity
				end
			end
		end
	end

	return 0, 1
end

function ChargingStation:update(dt)
	ChargingStation:superClass().update(self, dt)

	if self.loadTrigger ~= nil then
		local isActive = next(self.loadTrigger.fillableObjects) ~= nil

		if isActive then
			local color = self.chargeIndicatorColorEmpty
			local fillLevel, capacity = self:getChargeState()

			if fillLevel / capacity > 0.95 then
				color = self.chargeIndicatorColorFull
			end

			setShaderParameter(self.chargeIndicatorNode, "colorScale", color[1], color[2], color[3], color[4], false)

			self.chargeIndicatorLightColor = color
		end

		local blinkSpeed = self.loadTrigger.isLoading and self.chargeIndicatorBlinkSpeed or 0

		setShaderParameter(self.chargeIndicatorNode, "blinkSpeed", blinkSpeed, 0, 0, 0, false)
		setShaderParameter(self.chargeIndicatorNode, "lightControl", isActive and self.chargeIndicatorIntensity or 0, 0, 0, 0, false)

		if self.chargeIndicatorLight ~= nil then
			local alpha = 0

			if isActive then
				alpha = MathUtil.clamp(math.cos(blinkSpeed * getShaderTimeSec()) + 0.2, 0, 1)
			end

			setLightColor(self.chargeIndicatorLight, self.chargeIndicatorLightColor[1] * alpha, self.chargeIndicatorLightColor[2] * alpha, self.chargeIndicatorLightColor[3] * alpha)
		end

		if self.loadTrigger.isLoading then
			local allowDisplay = false

			if g_currentMission.controlPlayer and g_currentMission.player ~= nil then
				local distance = calcDistanceFrom(g_currentMission.player.rootNode, self.nodeId)

				if distance < self.interactionRadius then
					allowDisplay = true
				end
			elseif g_currentMission.controlledVehicle ~= nil then
				for _, object in pairs(self.loadTrigger.fillableObjects) do
					if object.object == g_currentMission.controlledVehicle then
						allowDisplay = true
					end
				end
			end

			if allowDisplay then
				local fillLevel, capacity = self:getChargeState()
				local fillLevelToFill = capacity - fillLevel
				local literPerSecond = self.loadTrigger.fillLitersPerMS * 1000
				local seconds = fillLevelToFill / literPerSecond

				if seconds >= 1 then
					local minutes = math.floor(seconds / 60)
					seconds = seconds - minutes * 60

					g_currentMission:addExtraPrintText(string.format(g_i18n:getText("info_chargeTime"), minutes, seconds, fillLevel / capacity * 100))
				end
			end
		end
	end

	self:raiseActive()
end
